function w=VSomp(x,S)
% VSomp     Orthogonal Matching Pursuit with QR decomposition, Vector Selection algorithm 
%  
% w=VSomp(x,S);
%-----------------------------------------------------------------------------------
% arguments:
%   w    - the weights where at most S of the elements are non-zero, size Kx1
%   x    - the vector to approximate, size Nx1
%   S    - number of vectors to select or non-zero weights 
%-----------------------------------------------------------------------------------
% global variables:
%   F    - the normalized dictionary (or F matrix), size NxK
%          (this matrix is called U in some papers describing OMP/ORMP/FOMP)
%   FF   - the inner products F'*F, size KxK
%-----------------------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 2003.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing 
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  12.06.2003  KS: function made
%-----------------------------------------------------------------------------------

global F FF

Mfile='VSomp';
[N,K]=size(F);

% **********************  INITIALIZE  **********************
r = zeros(S,K);
w = zeros(K,1);
T = 1:K;
e = ones(K,1);
u = ones(K,1);
c = x'*F;
n2x = x'*x;
n2xLim = n2x*1e-10;
% select the first frame vector
[cm,km] = max(abs(c));
s = 1;
J = km;  
T(km) = -1;  
r(1,km) = u(km);
n2x = n2x-cm*cm;

% **********************  THE MAIN LOOP  **********************
while ((s<S) & (n2x>n2xLim))
    for k=1:K
        if (T(k)>=0)
            r(s,k) = FF(km,k);
            for n=1:(s-1)
                r(s,k) = r(s,k)-r(n,km)*r(n,k);
            end
            if (u(km)~=0); r(s,k) = r(s,k)/u(km); end;
            c(k) = c(k)*u(k)-c(km)*r(s,k);  
            % use next line for OMP
            w(k) = c(k);     % use w here (instead of a new variable d)
            e(k) = e(k)-r(s,k)*r(s,k);
            u(k) = sqrt(abs(e(k)));  % abs kun i matlab!
            if (u(k)~=0); c(k) = c(k)/u(k); end;  
            % use next line for ORMP
            % w(k) = c(k);     % use w here (instead of a new variable d)
        end
    end
    w(J) = 0;  
    % select the next frame vector
    [cm,km] = max(abs(w));
    s = s+1;
    J(s) = km;
    T(km) = -1;
    r(s,km) = u(km);
    cm = c(km);
    n2x = n2x-cm*cm;
end  % ******** END OF MAIN LOOP **********************************

% ************ BACK-SUBSTITUTION *************
w = zeros(K,1);
for k=s:(-1):1
    Jk=J(k);
    for n=s:(-1):(k+1)
        c(Jk) = c(Jk)-c(J(n))*r(k,J(n));
    end
    if (r(k,Jk)~=0); c(Jk) = c(Jk)/r(k,Jk); end;
    w(Jk) = c(Jk);
end

return;
