function w=VSormp2(x,S)
% VSormp2   Order Recursive Matching Pursuit with QR decomposition, Vector Selection algorithm 
%           This function is more "Matlab-oriented" that VSormp and VSomp,
% here we use fewer loops and more of the Matlab capabilities, but even so I suppose 
% that the VSormp and VSomp functions are the better ones (at least with the newer versions 
% of Matlab that execute loops quite fast.) Also this function does not ned
% the global variable FF, the inner products of the frame vectors.
%  
% w=VSormp2(x,S);
%-----------------------------------------------------------------------------------
% arguments:
%   w    - the weights where at most S of the elements are non-zero, size Kx1
%   x    - the vector to approximate, size Nx1
%   S    - number of vectors to select or non-zero weights 
%-----------------------------------------------------------------------------------
% global variables:
%   F    - the normalized dictionary (or F matrix), size NxK
%          (this matrix is called U in some papers describing OMP/ORMP/FOMP)
%-----------------------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 2003.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing 
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  19.06.2003  KS: function made
%-----------------------------------------------------------------------------------

global F

Mfile='VSormp2';
[N,K]=size(F);

% **********************  INITIALIZE  **********************
R = zeros(S,K);
e = ones(K,1);
u = ones(K,1);
c = F'*x;   % Kx1
d = abs(c);
nx = x'*x;
nxLim = nx*1e-10;
i = 0;
I = [];
J = 1:K;

% **********************  THE MAIN LOOP  **********************
while 1
    % select the first frame vector
    [temp,k] = max(d); 
    i = i+1;
    I = [I,k];  
    J = setdiff(J,k);
    %
    R(i,k) = u(k);
    nx = nx-c(k)^2;
    %
    if ((i >= S) || (nx<nxLim)); break; end;     % ===================> EXIT MAIN LOOP
    %
    R(i,J) = F(:,k)'*F(:,J); 
    for n=1:(i-1)
        R(i,J) = R(i,J)-R(n,k)*R(n,J); 
    end
    R(i,J) = R(i,J)/u(k); 
    c(J) = c(J).*u(J)-c(k)*R(i,J)';
    % d(J) = abs(c(J));           % this is used for OMP
    e(J) = e(J)-R(i,J)'.^2;
    u(J) = sqrt(abs(e(J)));  
    c(J) = c(J)./u(J); 
    d(J) = abs(c(J));           % this is used for ORMP
    d(k) = 0;  
end  % ******** END OF MAIN LOOP **********************************

% ************ BACK-SUBSTITUTION *************
w = zeros(K,1);
%   the simple Matlab way
w(I)=R(1:i,I)\c(I);
%   or the detailed way
% for k=I; w(k)=c(k); end;
% for k=i:(-1):1
%     ik=I(k);
%     for n=i:(-1):(k+1)
%         w(ik)=w(ik)-w(I(n))*R(k,I(n));
%     end
%     w(ik)=w(ik)/R(k,ik);
% end

return;


