function w=VSps2(x,S,NoCombinations,g1,g2,g3)
% VSps2     Partial Search 2, Vector Selection algorithm 
%  
% w=VSps2(x,S,NoCombinations);
% w=VSps2(x,S,NoCombinations,g1,g2,g3);
%-----------------------------------------------------------------------------------
% arguments:
%   w    - the weights where at most S of the elements are non-zero, size Kx1
%   x    - the vector to approximate, size Nx1
%   S    - number of vectors to select or non-zero weights 
%   NoCombinations    - number of combinations to search 
%        Some special parameters for testing (default values are usually ok)
%   g1   - only d-values above this limit (relative to max(d)) is considered
%   g2   - and this will be given g2*(max number of comb. on this level, for max(d))
%   g3   - what is added before the floor function
%-----------------------------------------------------------------------------------
% global variables:
%   F    - the normalized dictionary (or F matrix), size NxK
%          (this matrix is called U in some papers describing OMP/ORMP/FOMP)
%   FF   - the inner products F'*F, size KxK
%-----------------------------------------------------------------------------------

%----------------------------------------------------------------------
% Copyright (c) 2003.  Karl Skretting.  All rights reserved.
% Hogskolen in Stavanger (Stavanger University), Signal Processing 
% Mail:  karl.skretting@tn.his.no   Homepage:  http://www.ux.his.no/~karlsk/
% 
% HISTORY:  dd.mm.yyyy
% Ver. 1.0  12.06.2003  KS: function made
% Ver. 1.1  23.06.2003  KS: some improvements
%-----------------------------------------------------------------------------------

global F FF
Display=0;

Mfile='VSps2';
if (nargin<4); g1 = 0.6;  end;
if (nargin<5); g2 = 0.5;  end;
if (nargin<6); g3 = 0.6;  end;
b1 = (1-g2)/(1-g1);
b2 = (g2-g1)/(1-g1);

% **********************  INITIALIZE  **********************
[N,K]=size(F);
R = zeros(S,K);
d = zeros(1,K);
e = ones(1,K);
u = ones(1,K);
ceu = zeros(S,3*K+1);
M = zeros(S,K);
J = 1:K;
I = 1:S;
i = 0;
c = x'*F;
d = abs(c);
nx = x'*x;
nxBest = nx;
nxLim = nx*1e-10;
%
Nedover=1;
IkkeVelg=0;
AntallSjekket=0;
if Display
    disp(' ');
    disp([Mfile,': using size(F)=',int2str(N),'x',int2str(K),', S=',int2str(S),', NoCombinations=',int2str(NoCombinations),'.']);
    disp(['Norm squared of x, nx=',num2str(nx),', and the limit is nxLim=',num2str(nxLim)]);
end

% **********************  THE MAIN LOOP  **********************
while 1
    if Nedover
        % ************  SELECT NEW FRAME VECTOR AND FIND COMBINATIONS TO SEARCH  ************
        if (IkkeVelg>0); d(IkkeVelg)=0; end;
        if (i==0)
            m=NoCombinations;
        else
            m=M(i,I(i));  % den sist valgte
        end
        i=i+1;  
        if (i==S); m=1; end;
        %
        [cm,km] = max(d);
        if (m==1)
            no = 1;
            M(i,:) = zeros(1,K);
            M(i,km) = 1;
        elseif ((i+1)==S)    % use the m largest
            [a1,temp]=sort(-d);
            M(i,:) = zeros(1,K);
            no=min([K-i-1,m]);
            M(i,temp(1:no))=1;
        else   
            % ************  FIND WHICH COMBINATIONS TO SEARCH, MAIN CASE  **********************
            d = d*b1/cm+b2;
            a1 = 0;
            for k=1:K
                if ((d(k)<g1) | (J(k)<0))
                    d(k) = 0;
                else
                    a1 = a1+d(k);
                end
            end
            if (a1>0)
                a1 = m/a1;    
                M(i,:) = floor(d*a1+g3);
                % some exceptions are handled
                k = sum(M(i,:));
                if (k==0)
                    M(i,km)=m;
                    k=m;
                end
                while (k>m)
                    % M(i,km) = M(i,km)+m-k; % the direct way
                    ii=find(M(i,:));
                    if (length(ii)==0)
                        disp([Mfile,': Find children part ERROR (?), ii']);
                    end
                    [a2,i3]=min(d(ii));
                    if M(i,ii(i3))<1
                        t=[Mfile,': Find children part ERROR (?), M'];
                        t=[t,', max(M(',int2str(i),',:))=',int2str(max(M(i,:)))];
                        t=[t,', sum(M(',int2str(i),',:))=',int2str(sum(M(i,:)))];
                        t=[t,', M(',int2str(i),',',int2str(ii(i3)),')=',int2str(M(i,ii(i3)))];
                        t=[t,', m=',int2str(m)];
                        t=[t,', length(ii)=',int2str(length(ii))];
                        t=[t,', i3=',int2str(i3)];
                        disp(t);
                        k=m;
                    else
                        M(i,ii(i3))=M(i,ii(i3))-1;
                        d(ii(i3))=0;
                        k = sum(M(i,:));
                    end
                end
                if (M(i,km)<1)
                    t=[Mfile,': Find children part OBS, M(',int2str(i),',',int2str(km),')=',int2str(M(i,km))];
                    t=[t,', max(M(',int2str(i),',:))=',int2str(max(M(i,:)))];
                    t=[t,', sum(M(',int2str(i),',:))=',int2str(sum(M(i,:)))];
                    t=[t,', m=',int2str(m)];
                    disp(t);
                    M(i,km)=max(M(i,:))+1;
                end
            else
                % this should not happen
                disp([Mfile,': Find children part ERROR (?), a1=',num2str(a1)]);
            end
            temp = find(M(i,:));   % temp is also used in next Display
            no = sum(temp>0);
            %
            if (Display > 1)
                disp([blanks(3*i),'m=',int2str(m),' gir no=',int2str(no),', km=',int2str(km)]);
                for i3=1:length(temp)
                    t=[blanks(3*i),' M(',int2str(i),',',int2str(temp(i3)),')=',int2str(M(i,temp(i3)))];
                    t=[t,' d(',int2str(temp(i3)),')=',num2str(d(temp(i3)))];
                    t=[t,' c(',int2str(temp(i3)),')=',num2str(c(temp(i3)))];
                    disp(t);
                end
            end
            % ************  HAVE FOUND COMBINATIONS TO SEARCH, END OF MAIN CASE  **************
        end
        % **** STORE/PUSH some variables ***
        if (m>1)  
            ceu(i,:)=[c,e,u,nx];
        end
    end
    Nedover=1;
    %
    % **** SELECT frame vector number i ***
    I(i)=km; J(km)=-1;
    R(i,km)=u(km);
    nx = nx-cm*cm;
    if (Display > 1)
        disp(['i=',int2str(i),', nx=',num2str(nx),', cm=',num2str(cm)]);
    end
    %
    % **** CHECK if norm of residual is small enough ***
    if (nx < nxLim)
        if Display
            disp([Mfile,': finish because nx < nxLim']);
        end
        % ****************** BACK-SUBSTITUTION *****************
        w = zeros(K,1);
%         w(I(1:i)) = R(1:i,I(1:i))\(c(I(1:i))');
        for k=i:(-1):1
            ik=I(k);
            for n=i:(-1):(k+1)
                c(ik) = c(ik)-c(I(n))*R(k,I(n));
            end
            if (R(k,ik)~=0); c(ik) = c(ik)/R(k,ik); end;
            w(ik) = c(ik);
        end
        break;   % ************** ==========================>  EXIT MAIN LOOP
    end
    %
    % ******** CHECK if more frame vectors may be selected **************
    if (i<S) 
        % ****************** UPDATE R(i,:), c, e, and u *****************
        for k=1:K
            if (J(k)>=0)
                R(i,k) = FF(km,k);
                for n=1:(i-1)
                    R(i,k) = R(i,k)-R(n,km)*R(n,k);
                end
                if (u(km)~=0); R(i,k) = R(i,k)/u(km); end;
                c(k) = c(k)*u(k)-c(km)*R(i,k);
                % d(k) = abs(c(k));  % for OMP
                e(k) = e(k)-R(i,k)*R(i,k);
                u(k) = sqrt(abs(e(k)));  
                if (u(k)~=0); c(k) = c(k)/u(k); end;
            end
        end
        d=abs(c);
        d(I(1:i))=0;
    else
        % *** no more vectors may be selected ***
        AntallSjekket=AntallSjekket+1;
        if (Display > 1)
            t=['Combination ',int2str(AntallSjekket)];
            t=[t,', sqrt(nx)=',num2str(sqrt(nx),'%7.4f'),'  I = '];
            for ss=1:S; t=[t,'  ',int2str(I(ss))]; end;
            disp(t);
        end
        %
        % *** is this combination the best so far ***
        if (nx < nxBest)
            % ****************** BACK-SUBSTITUTION *****************
            w = zeros(K,1);
%             w(I(1:i)) = R(1:i,I(1:i))\(c(I(1:i))');
            for k=i:(-1):1
                ik=I(k);
                for n=i:(-1):(k+1)
                    c(ik) = c(ik)-c(I(n))*R(k,I(n));
                end
                if (R(k,ik)~=0); c(ik) = c(ik)/R(k,ik); end;
                w(ik) = c(ik);
            end
            % *** and store the best 
            nxBest = nx; 
            if Display
                t=['Combination ',int2str(AntallSjekket)];
                t=[t,' improved error squared to ',num2str(nxBest,'%7.4f')];
                disp(t);
            end
        end
        %
        if (AntallSjekket==NoCombinations)
            if Display
                disp([Mfile,': Normal exit (NoCombinations), combination searched is ',int2str(AntallSjekket)]);
            end
            break;   % ************** ==========================>  EXIT MAIN LOOP
        end
        %
        % *** prepare to search another combination one (or more) level(i) up
        i=i-1;
        J(km)=km;
        while (i>0)
            IkkeVelg=I(i);
            M(i,I(i))=0;
            J(I(i))=I(i);
            [a1,km]=max(M(i,:));
            if (a1>0) break; end;
            i=i-1;
        end
        %
        if (i<=0) 
            if Display
                disp([Mfile,': Normal exit? (i), ',int2str(AntallSjekket),' combinations was searched.']);
            end
            break;   % ************** ==========================>  EXIT MAIN LOOP
        end
        %
        % **** RETRIEVE/POP some variables ***
        c=ceu(i,1:K);
        e=ceu(i,(K+1):(2*K));
        u=ceu(i,(2*K+1):(3*K));
        nx=ceu(i,(3*K+1):length(ceu));
        %
        cm=c(km);
        %
        Nedover=0;
    end  % *** END OF CHECK if more frame vectors may be selected ***
    %
end  % ******** END OF MAIN LOOP **********************************

return;


